BuiltinTy

Enum BuiltinTy 

pub enum BuiltinTy {
    Tuple,
    Box,
    Str,
}
Builtin types identifiers.

WARNING: for now, all the built-in types are covariant in the generic parameters (if there are). Adding types which don’t satisfy this will require to update the code abstracting the signatures (to properly take into account the lifetime constraints).

TODO: update to not hardcode the types (except Box maybe) and be more modular. TODO: move to builtins.rs?

Variants



Tuple

Tuple type.



Box

Boxes are de facto a primitive type.



Str

Primitive type

Implementations

Source

impl BuiltinTy

pub fn is_tuple(&self) -> bool

pub fn is_box(&self) -> bool

pub fn is_str(&self) -> bool

Source

impl BuiltinTy

pub fn as_tuple(&self) -> Option<()>

pub fn as_box(&self) -> Option<()>

pub fn as_str(&self) -> Option<()>

Source

impl BuiltinTy

pub fn as_tuple_mut(&mut self) -> Option<()>

pub fn as_box_mut(&mut self) -> Option<()>

pub fn as_str_mut(&mut self) -> Option<()>

Source

impl BuiltinTy

pub fn variant_name(&self) -> &'static str

Source

impl BuiltinTy

pub fn get_name(self) -> Name