pub enum TyKind {
Show 15 variants
Scalar(ScalarTy),
Array(Ty, ConstantExpr, Option<TraitRef>),
Slice(Ty, Option<TraitRef>),
Adt(TypeDeclRef),
Ref(Region, Ty, RefKind),
RawPtr(Ty, RefKind),
FnDef(RegionBinder<FnPtr>),
FnPtr(RegionBinder<FunSig>),
DynTrait(DynPredicate),
Pattern(Ty, TypePattern),
Never,
TypeVar(TypeDbVar),
TraitType(TraitRef, AssocTypeId, GenericArgs),
PtrMetadata(Ty),
Error(String),
}Expand description
A type.
This is interned as Ty, making it cheap to clone and compare.
Variants
Scalar(ScalarTy)
A scalar (integers, floats, char, or bool).
Array(Ty, ConstantExpr, Option<TraitRef>)
An array [T; N]. The third field is the proof that T: Sized; it is absent with
--hide-marker-traits.
Slice(Ty, Option<TraitRef>)
A slice [T]. The second field is the proof that T: Sized; it is absent with
--hide-marker-traits.
Adt(TypeDeclRef)
An ADT: structs, enums, unions, as well as tuples and str.
Ref(Region, Ty, RefKind)
A reference: &T or &mut T.
RawPtr(Ty, RefKind)
A raw pointer.
FnDef(RegionBinder<FnPtr>)
The unique type associated with each function item. Each function item is given a unique
type that has the function’s early-bound generics. This type is not generally nameable in
Rust; it’s a ZST (there’s a unique value), and a value of that type can be cast to a
function pointer or passed to functions that expect FnOnce/FnMut/Fn parameters.
There’s a binder here because charon function items take both early and late-bound lifetimes as arguments; given that the type we’re pointing to is polymorphic in the late-bound variables, we need to bind them here.
// `'a` is early-bound, 'b is late-bound.
fn foo<'a, 'b>(x: &'a u32, y: &'b u32)
where u32: 'b
{}For rustc, there’s a ZST foo<'a>, that can be cast to a for<'b> fn(&'a u32, &'b u32)
function pointer.
For charon, there’s an item foo<'a, 'b>, and the FnDef item that corresponds to rustc’s
foo<'a> is represented as FnDef(for<'b> foo<'a, 'b>).
FnPtr(RegionBinder<FunSig>)
Function pointer type. This is a literal pointer to a region of memory that contains a callable function.
A function pointer can have lifetime generics, e.g. for<'a> fn(&'a mut u32) -> &'a u32,
hence the binder.
DynTrait(DynPredicate)
dyn Trait: erased value known to implement Trait. A pointer to it will carry a vtable
pointer that stores the methods that can be called on this value.
Pattern(Ty, TypePattern)
A pattern type: a type that is representationally identical to its base type, except the only valid values are the ones that match the pattern.
Never
The never type, the canonical uninhabited type.
TypeVar(TypeDbVar)
A type variable.
TraitType(TraitRef, AssocTypeId, GenericArgs)
A trait associated type: <T as Trait>::AssocType<Args>.
PtrMetadata(Ty)
The type of pointer metadata for the given type; e.g. for [T], this type is usize. The
way to write this type in Rust is <X as core::ptr::Pointee>::Metadata.
Error(String)
A type that could not be computed or was incorrect.
Implementations
Sourceimpl TyKind
impl TyKind
pub fn is_scalar(&self) -> bool
pub fn is_array(&self) -> bool
pub fn is_slice(&self) -> bool
pub fn is_adt(&self) -> bool
pub fn is_ref(&self) -> bool
pub fn is_raw_ptr(&self) -> bool
pub fn is_fn_def(&self) -> bool
pub fn is_fn_ptr(&self) -> bool
pub fn is_dyn_trait(&self) -> bool
pub fn is_pattern(&self) -> bool
pub fn is_never(&self) -> bool
pub fn is_type_var(&self) -> bool
pub fn is_trait_type(&self) -> bool
pub fn is_ptr_metadata(&self) -> bool
pub fn is_error(&self) -> bool
Sourceimpl TyKind
impl TyKind
pub fn as_scalar(&self) -> Option<&ScalarTy>
pub fn as_array(&self) -> Option<(&Ty, &ConstantExpr, &Option<TraitRef>)>
pub fn as_slice(&self) -> Option<(&Ty, &Option<TraitRef>)>
pub fn as_adt(&self) -> Option<&TypeDeclRef>
pub fn as_ref(&self) -> Option<(&Region, &Ty, &RefKind)>
pub fn as_raw_ptr(&self) -> Option<(&Ty, &RefKind)>
pub fn as_fn_def(&self) -> Option<&RegionBinder<FnPtr>>
pub fn as_fn_ptr(&self) -> Option<&RegionBinder<FunSig>>
pub fn as_dyn_trait(&self) -> Option<&DynPredicate>
pub fn as_pattern(&self) -> Option<(&Ty, &TypePattern)>
pub fn as_never(&self) -> Option<()>
pub fn as_type_var(&self) -> Option<&TypeDbVar>
pub fn as_trait_type(&self) -> Option<(&TraitRef, &AssocTypeId, &GenericArgs)>
pub fn as_ptr_metadata(&self) -> Option<&Ty>
pub fn as_error(&self) -> Option<&String>
Sourceimpl TyKind
impl TyKind
pub fn as_scalar_mut(&mut self) -> Option<&mut ScalarTy>
pub fn as_array_mut( &mut self, ) -> Option<(&mut Ty, &mut ConstantExpr, &mut Option<TraitRef>)>
pub fn as_slice_mut(&mut self) -> Option<(&mut Ty, &mut Option<TraitRef>)>
pub fn as_adt_mut(&mut self) -> Option<&mut TypeDeclRef>
pub fn as_ref_mut(&mut self) -> Option<(&mut Region, &mut Ty, &mut RefKind)>
pub fn as_raw_ptr_mut(&mut self) -> Option<(&mut Ty, &mut RefKind)>
pub fn as_fn_def_mut(&mut self) -> Option<&mut RegionBinder<FnPtr>>
pub fn as_fn_ptr_mut(&mut self) -> Option<&mut RegionBinder<FunSig>>
pub fn as_dyn_trait_mut(&mut self) -> Option<&mut DynPredicate>
pub fn as_pattern_mut(&mut self) -> Option<(&mut Ty, &mut TypePattern)>
pub fn as_never_mut(&mut self) -> Option<()>
pub fn as_type_var_mut(&mut self) -> Option<&mut TypeDbVar>
pub fn as_trait_type_mut( &mut self, ) -> Option<(&mut TraitRef, &mut AssocTypeId, &mut GenericArgs)>
pub fn as_ptr_metadata_mut(&mut self) -> Option<&mut Ty>
pub fn as_error_mut(&mut self) -> Option<&mut String>
Sourceimpl TyKind
impl TyKind
pub fn to_scalar(self) -> Option<ScalarTy>
pub fn to_array(self) -> Option<(Ty, ConstantExpr, Option<TraitRef>)>
pub fn to_slice(self) -> Option<(Ty, Option<TraitRef>)>
pub fn to_adt(self) -> Option<TypeDeclRef>
pub fn to_ref(self) -> Option<(Region, Ty, RefKind)>
pub fn to_raw_ptr(self) -> Option<(Ty, RefKind)>
pub fn to_fn_def(self) -> Option<RegionBinder<FnPtr>>
pub fn to_fn_ptr(self) -> Option<RegionBinder<FunSig>>
pub fn to_dyn_trait(self) -> Option<DynPredicate>
pub fn to_pattern(self) -> Option<(Ty, TypePattern)>
pub fn to_never(self) -> Option<()>
pub fn to_type_var(self) -> Option<TypeDbVar>
pub fn to_trait_type(self) -> Option<(TraitRef, AssocTypeId, GenericArgs)>
pub fn to_ptr_metadata(self) -> Option<Ty>
pub fn to_error(self) -> Option<String>
Sourceimpl TyKind
impl TyKind
pub fn into_ty(self) -> Ty
pub fn is_usize(&self) -> bool
pub fn is_unsigned_scalar(&self) -> bool
pub fn is_signed_scalar(&self) -> bool
pub fn is_str(&self) -> bool
pub fn is_tuple(&self) -> bool
pub fn as_adt_id(&self) -> Option<TypeDeclId>
pub fn as_box(&self) -> Option<&Ty>
pub fn as_box_mut(&mut self) -> Option<&mut Ty>
pub fn builtin_deref(&self) -> Option<&Ty>
pub fn builtin_deref_mut(&mut self) -> Option<&mut Ty>
pub fn as_array_or_slice(&self) -> Option<&Ty>
pub fn as_array_or_slice_mut(&mut self) -> Option<&mut Ty>